// -*- C++ -*-
/*!
 * @file  EchoMonitor.cpp
 * @brief EchoMonitor component
 * @date
 * @author Kazuo Nakayoshi <kazuo.nakayoshi@kek.jp>
 *
 * Copyright (C) 2008
 *     Kazuo Nakayoshi
 *     Electronics System Group,
 *     KEK, Japan.
 *     All rights reserved.
 *
 */

#include "EchoMonitor.h"
#include "stringUtils.h"

// Module specification
static const char* echomonitor_spec[] =
{
    "implementation_id", "EchoMonitor",
    "type_name",         "EchoMonitor",
    "description",       "EchoMonitor component",
    "version",           "1.0",
    "vendor",            "Kazuo Nakayoshi, KEK",
    "category",          "example",
    "activity_type",     "DataFlowComponent",
    "max_instance",      "10",
    "language",          "C++",
    "lang_type",         "compile",
    ""
};

EchoMonitor::EchoMonitor(RTC::Manager* manager)
    : DAQMW::MlfComponent(manager),
        m_InPort("echomonitor_in",   m_in_data),
        //m_OutPort("echomonitor_out", m_out_data),

        m_in_status(BUF_SUCCESS),
        //m_out_status(BUF_SUCCESS),

        m_debug(false)
{
    // Registration: InPort/OutPort/Service

    // Set InPort buffers
    registerInPort ("echomonitor_in",   m_InPort);
    //registerOutPort("echomonitor_out", m_OutPort);

    init_command_port();
    init_state_table( );
    set_comp_name(COMP_NONAME);
}

EchoMonitor::~EchoMonitor()
{
}


RTC::ReturnCode_t EchoMonitor::onInitialize()
{
    if (m_debug) {
        std::cerr << "EchoMonitor::onInitialize()" << std::endl;
    }

    return RTC::RTC_OK;
}

RTC::ReturnCode_t EchoMonitor::onExecute(RTC::UniqueId ec_id)
{
    daq_do();

    return RTC::RTC_OK;
}

int EchoMonitor::daq_dummy()
{
    return 0;
}

int EchoMonitor::daq_configure()
{
    std::cerr << "*** EchoMonitor::configure" << std::endl;

    ::NVList* paramList;
    paramList = m_daq_service0.getCompParams();
    parse_params(paramList);

    return 0;
}

int EchoMonitor::parse_params(::NVList* list)
{

    std::cerr << "param list length:" << (*list).length() << std::endl;

    int length = (*list).length();
    for (int i = 0; i < length; i++) {
        if (m_debug) {
            std::cerr << "NVList[" << (*list)[i].name
                      << ","<< (*list)[i].value << "]" << std::endl;
        }
    }

    return 0;
}


int EchoMonitor::daq_unconfigure()
{
    std::cerr << "*** EchoMonitor::unconfigure" << std::endl;

    return 0;
}

int EchoMonitor::daq_start()
{
    std::cerr << "*** EchoMonitor::start" << std::endl;

    m_in_status  = BUF_SUCCESS;
    m_out_status = BUF_SUCCESS;

    return 0;
}

int EchoMonitor::daq_stop()
{
    std::cerr << "*** EchoMonitor::stop" << std::endl;

    reset_InPort();

    return 0;
}

int EchoMonitor::daq_pause()
{
    std::cerr << "*** EchoMonitor::pause" << std::endl;
    return 0;
}

int EchoMonitor::daq_resume()
{
    std::cerr << "*** EchoMonitor::resume" << std::endl;
    return 0;
}

int EchoMonitor::daq_run()
{
    if (m_debug) {
        std::cerr << "*** EchoMonitor::run" << std::endl;
    }

    m_in_status = m_InPort.read(m_in_data);
    if ((m_in_status == BUF_TIMEOUT) && check_trans_lock()) {
        set_trans_unlock();
        return 0;
    }
    else if (m_in_status == BUF_TIMEOUT) {
        return 0;
    }
    else if (m_in_status == BUF_FATAL) {
        fatal_error_report(USER_ERROR1, -1);
        return 0;
    }

    /////////////// Get Data Length ///////////////
    unsigned int block_byte_size = m_in_data.data.length();
    unsigned int event_byte_size = block_byte_size - HEADER_BYTE_SIZE - FOOTER_BYTE_SIZE;

    /////////////// Check Header and Footer ///////////////
    unsigned char header[HEADER_BYTE_SIZE];
    unsigned char footer[FOOTER_BYTE_SIZE];
    for (unsigned int i = 0; i < HEADER_BYTE_SIZE; i++) {
        header[i] = m_in_data.data[i];
    }
    if (check_header(header, event_byte_size) == false) {
        std::cerr << "### ERROR: header invalid in EchoMonitor" << std::endl;
        fatal_error_report(HEADER_DATA_MISMATCH, -1);
        return 0;
    }
    for (unsigned int i = 0; i < FOOTER_BYTE_SIZE; i++) {
        footer[i] = m_in_data.data[block_byte_size - FOOTER_BYTE_SIZE + i];
    }
    if (check_footer(footer, m_loop) == false) {
        std::cerr << "### ERROR: footer invalid in EchoMonitor" << std::endl;
        fatal_error_report(FOOTER_DATA_MISMATCH, -1);
        return 0;
    }

    ////////// Extract each event data and print to STDERR //////////
    unsigned int *event_data;
    for (unsigned int i = HEADER_BYTE_SIZE; i < block_byte_size - FOOTER_BYTE_SIZE; i += EVENT_BYTE_SIZE) {
        event_data = (unsigned int *) &m_in_data.data[i];
        std::cerr << "Event Data: " << *event_data << std::endl;
    }
    std::cerr << "*** Data Extraction End" << std::endl;

    ////////// One daq_run() with reading data almost done //////////
    m_loop++;
    m_total_event += event_byte_size / EVENT_BYTE_SIZE;

    if (check_trans_lock()) {  // got stop command
        set_trans_unlock();
        return 0;
    }

    return 0;
}

int EchoMonitor::reset_InPort()
{
    TimedOctetSeq dummy_data;

    int ret = BUF_SUCCESS;
    while (ret == BUF_SUCCESS) {
        ret = m_InPort.read(dummy_data);
    }
    std::cerr << "*** Monitor::InPort flushed\n";
    return 0;
}

extern "C"
{

    void EchoMonitorInit(RTC::Manager* manager)
    {
    RTC::Properties profile(echomonitor_spec);
    manager->registerFactory(profile,
                 RTC::Create<EchoMonitor>,
                 RTC::Delete<EchoMonitor>);
    }
};
